/********************************************************************************
 *   dci710usb -- usb function switch for samsung sgh i710 device.              *
 *                                                                              * 
 *   Copyright (C) 2004-2007 dci (dev@cpp.in)                                   *
 *                                                                              *
 *   This program is free software; you can redistribute it and/or              *
 *   modify it under the terms of the GNU General Public License version 2      *
 *   as published by the Free Software Foundation.                              *
 *                                                                              *
 *   This program is distributed in the hope that it will be useful,            *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of             *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU          *
 *   General Public License version 2 for more details.                         *
 *                                                                              *
 *   You should have received a copy of the GNU General Public License          *
 *   version 2 along with this library; if not, write to the Free Software      *
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA  *
 ********************************************************************************
 */


#include <windows.h>
#include <winioctl.h>
#include <aygshell.h>
#include "resource.h"


/* UFN STUFF */
#define UFN_DRIVER_NAME_LEN   378

#define IOCTL_UFN_SETSTATE       0x80030000
#define IOCTL_UFN_DETACH         0x80030008
#define IOCTL_UFNCTL_START       0x00220004
#define IOCTL_UFNCTL_READ        0x00220008
#define IOCTL_UFNCTL_ACTIVE_GET  0x0022000C
#define IOCTL_UFNCTL_ACTIVE_SET  0x00220010
#define IOCTL_UFNCTL_DEFAULT_GET 0x00220014
#define IOCTL_UFNCTL_DEFAULT_SET 0x00220018
// {E2BDC372-598F-4619-BC50-54B3F7848D35}
static const GUID UFNCTL_GUID = { 0xE2BDC372, 0x598F, 0x4619, { 0xBC, 0x50, 0x54, 0xB3, 0xF7, 0x84, 0x8D, 0x35 } };


/* my stuff */
#define NONE L"(none)"


HANDLE hUFN    = INVALID_HANDLE_VALUE;
HANDLE hUFNCtl = INVALID_HANDLE_VALUE;

wchar_t szActiveDriver[UFN_DRIVER_NAME_LEN];
wchar_t szDefaultDriver[UFN_DRIVER_NAME_LEN];

/* dialog proc */
BOOL DlgProc ( HWND hDlg, UINT nMsg, WPARAM wParam, LPARAM lParam )
{
	if ( WM_INITDIALOG == nMsg )
	{
		DWORD readlen;
		/* fullscreen, ok */
		SHINITDLGINFO l_ShIDI;
		l_ShIDI.dwMask = SHIDIM_FLAGS;
		l_ShIDI.dwFlags = SHIDIF_DONEBUTTON | SHIDIF_SIPDOWN | SHIDIF_SIZEDLGFULLSCREEN;
		l_ShIDI.hDlg = hDlg;
		SHInitDialog(&l_ShIDI);
		/* menubar */
		SHMENUBARINFO mbi;
		memset ( &mbi, 0, sizeof ( mbi ) );
		mbi.cbSize = sizeof ( mbi );
		mbi.hwndParent = hDlg;
		mbi.nToolBarId = IDR_MAINMENU;
		mbi.hInstRes = ( HINSTANCE ) lParam;
		mbi.dwFlags |= SHCMBF_HMENU;
		SHCreateMenuBar ( &mbi );
		/* prepare comboboxes */
		SendDlgItemMessage ( hDlg, IDC_ACTIVE, CB_ADDSTRING, ( WPARAM ) 0, ( LPARAM ) NONE );
		SendDlgItemMessage ( hDlg, IDC_DEFAULT, CB_ADDSTRING, ( WPARAM ) 0, ( LPARAM ) NONE );
		/* read list */
		wchar_t name[UFN_DRIVER_NAME_LEN];
		DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_START, NULL, NULL, NULL, NULL, NULL, NULL );
		while ( DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_READ, NULL, NULL, name, sizeof ( name ), &readlen, NULL ) )
		{
			SendDlgItemMessage ( hDlg, IDC_ACTIVE, CB_ADDSTRING, ( WPARAM ) 0, ( LPARAM ) name );
			SendDlgItemMessage ( hDlg, IDC_DEFAULT, CB_ADDSTRING, ( WPARAM ) 0, ( LPARAM ) name );
		}
		/* get selection */
		DWORD sel = 0;
		DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_ACTIVE_GET, NULL, NULL, szActiveDriver, sizeof ( szActiveDriver ), &readlen, NULL );
		sel = SendDlgItemMessage ( hDlg, IDC_ACTIVE, CB_FINDSTRING, ( WPARAM ) 0, ( LPARAM ) szActiveDriver );
		if ( CB_ERR == sel ) sel = 0;
		SendDlgItemMessage ( hDlg, IDC_ACTIVE, CB_SETCURSEL, ( WPARAM ) sel, ( LPARAM ) 0 );
		//
		DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_DEFAULT_GET, NULL, NULL, szDefaultDriver, sizeof ( szDefaultDriver ), &readlen, NULL );
		sel = SendDlgItemMessage ( hDlg, IDC_DEFAULT, CB_FINDSTRING, ( WPARAM ) 0, ( LPARAM ) szDefaultDriver );
		if ( CB_ERR == sel ) sel = 0;
		SendDlgItemMessage ( hDlg, IDC_DEFAULT, CB_SETCURSEL, ( WPARAM ) sel, ( LPARAM ) 0 );
		/* */
		return TRUE;
	}
	else if ( WM_COMMAND == nMsg )
	{
		WORD wID = LOWORD(wParam);
		/* */
		if ( IDOK == wID )
		{
			DWORD readlen;
			wchar_t name[UFN_DRIVER_NAME_LEN];
			/* active */
			GetDlgItemText ( hDlg, IDC_ACTIVE, name, UFN_DRIVER_NAME_LEN );
			if ( _wcsicmp ( name, szActiveDriver ) )
			{
				DWORD dummy = 0;
				//DeviceIoControl ( hUFN, IOCTL_UFN_SETSTATE, &dummy, sizeof ( dummy ), NULL, 0, &readlen, NULL );
				dummy = 0;
				DeviceIoControl ( hUFN, IOCTL_UFN_DETACH, &dummy, sizeof ( dummy ), NULL, 0, &readlen, NULL );
				DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_ACTIVE_SET, name, /* REQUIRED! ( buggy impl? ) */ 0x100, NULL, NULL, NULL, NULL );
				dummy = 1;
				//DeviceIoControl ( hUFN, IOCTL_UFN_SETSTATE, &dummy, sizeof ( dummy ), NULL, 0, &readlen, NULL );
			}
			/* default */
			GetDlgItemText ( hDlg, IDC_DEFAULT, name, UFN_DRIVER_NAME_LEN );
			if ( _wcsicmp ( name, szDefaultDriver ) )
			{
				DeviceIoControl ( hUFNCtl, IOCTL_UFNCTL_DEFAULT_SET, name, /* REQUIRED! ( buggy impl? ) */ 0x100, NULL, NULL, NULL, NULL );
			}
			/* */
			EndDialog ( hDlg, 1 );
			return TRUE;
		}
		else if ( IDCANCEL == wID )
		{
			EndDialog ( hDlg, 0 );
			return TRUE;
		}
	}
	/* */
	return FALSE;
}

/* entrypoint */
int WinMain ( HINSTANCE hInst, HINSTANCE hPrevInst, LPTSTR lpCmdLine, int nCmdShow )
{
	/* try to open UFN */	
	hUFN = CreateFile ( L"UFN1:", GENERIC_READ | GENERIC_WRITE, 0, NULL, OPEN_EXISTING, 0, NULL );
	/* try to open UFNCtl */
	DEVMGR_DEVICE_INFORMATION ddictl;
	ddictl.dwSize = sizeof ( ddictl );
	HANDLE ctlsearch = FindFirstDevice ( DeviceSearchByGuid, &UFNCTL_GUID, &ddictl );
	if ( INVALID_HANDLE_VALUE != ctlsearch )
	{
		hUFNCtl = CreateFile ( ddictl.szBusName, GENERIC_READ, FILE_SHARE_READ, NULL, OPEN_EXISTING, NULL, NULL );
		FindClose ( ctlsearch );
	}
	/* success? */
	if ( ( INVALID_HANDLE_VALUE != hUFN ) && ( INVALID_HANDLE_VALUE != hUFNCtl ) )
	{
		/* write reg update */
		HKEY hkWrite;
		if ( ERROR_SUCCESS == RegOpenKeyEx( HKEY_LOCAL_MACHINE, L"Drivers\\USB\\FunctionDrivers\\Mass_Storage_Class", 0, NULL, &hkWrite ) )
		{
			RegSetValueEx ( hkWrite, L"Manufacturer", 0, REG_SZ, ( BYTE * ) L"Samsung", 14 );
			RegSetValueEx ( hkWrite, L"Product", 0, REG_SZ, ( BYTE * ) L"dci710usb", 18 );
			RegCloseKey ( hkWrite );
		}
		//
		if ( ERROR_SUCCESS == RegOpenKeyEx( HKEY_LOCAL_MACHINE, L"Drivers\\USB\\FunctionDrivers\\USBSER_Class", 0, NULL, &hkWrite ) )
		{
			RegSetValueEx ( hkWrite, L"Manufacturer", 0, REG_SZ, ( BYTE * ) L"Samsung", 14 );
			RegSetValueEx ( hkWrite, L"Product", 0, REG_SZ, ( BYTE * ) L"dci710usb", 18 );
			RegCloseKey ( hkWrite );
		}
		/* run dialog */
		DialogBoxParam ( hInst, MAKEINTRESOURCE(IDD_MAIN), NULL, DlgProc, ( LPARAM ) hInst );
		/* flush registry */
		RegFlushKey ( HKEY_CLASSES_ROOT );
		RegFlushKey ( HKEY_LOCAL_MACHINE );
		RegFlushKey ( HKEY_CURRENT_USER );
		RegFlushKey ( HKEY_USERS );
	}
	else
	{
		MessageBox ( NULL, L"Unknown USB driver/No privs!", L"dci", MB_TOPMOST | MB_ICONEXCLAMATION );		
	}
	/* close */
	if ( INVALID_HANDLE_VALUE != hUFNCtl )
	{
		CloseHandle ( hUFNCtl );
		hUFNCtl = INVALID_HANDLE_VALUE;
	}
	if ( INVALID_HANDLE_VALUE != hUFN )
	{
		CloseHandle ( hUFN );
		hUFN = INVALID_HANDLE_VALUE;
	}
	/* */
	return 0;
}
